# Autogenerated by nbdev

d = { 'settings': { 'branch': 'mainx',
                'doc_baseurl': '/fastbet',
                'doc_host': 'https://real-analytics-rd.github.io',
                'git_url': 'https://github.com/real-analytics-rd/fastbet',
                'lib_path': 'fastbet'},
  'syms': { 'fastbet.config.localconfig': {},
            'fastbet.config.mongo': { 'fastbet.config.mongo.db_connect': ('Config/mongo.html#db_connect', 'fastbet/config/mongo.py'),
                                      'fastbet.config.mongo.mongo_init': ('Config/mongo.html#mongo_init', 'fastbet/config/mongo.py')},
            'fastbet.core': {'fastbet.core.foo': ('core.html#foo', 'fastbet/core.py')},
            'fastbet.datastructure.data_extractor': { 'fastbet.datastructure.data_extractor.data_aggregator': ( 'dataStrcuture/data_extractor.html#data_aggregator',
                                                                                                                'fastbet/datastructure/data_extractor.py')},
            'fastbet.datastructure.game_features': { 'fastbet.datastructure.game_features.GameFeatures': ( 'dataStrcuture/game_features.html#gamefeatures',
                                                                                                           'fastbet/datastructure/game_features.py'),
                                                     'fastbet.datastructure.game_features.GameFeatures.get_all_games': ( 'dataStrcuture/game_features.html#gamefeatures.get_all_games',
                                                                                                                         'fastbet/datastructure/game_features.py'),
                                                     'fastbet.datastructure.game_features.GameFeatures.get_game': ( 'dataStrcuture/game_features.html#gamefeatures.get_game',
                                                                                                                    'fastbet/datastructure/game_features.py')},
            'fastbet.datastructure.odds': { 'fastbet.datastructure.odds.MarketOdds': ( 'dataStrcuture/odds.html#marketodds',
                                                                                       'fastbet/datastructure/odds.py'),
                                            'fastbet.datastructure.odds.MarketOdds.get_latest': ( 'dataStrcuture/odds.html#marketodds.get_latest',
                                                                                                  'fastbet/datastructure/odds.py'),
                                            'fastbet.datastructure.odds.MarketOdds.get_odds_features': ( 'dataStrcuture/odds.html#marketodds.get_odds_features',
                                                                                                         'fastbet/datastructure/odds.py')},
            'fastbet.datastructure.team_lineup': { 'fastbet.datastructure.team_lineup.Player': ( 'dataStrcuture/team_lineup.html#player',
                                                                                                 'fastbet/datastructure/team_lineup.py'),
                                                   'fastbet.datastructure.team_lineup.StartingPlayer': ( 'dataStrcuture/team_lineup.html#startingplayer',
                                                                                                         'fastbet/datastructure/team_lineup.py'),
                                                   'fastbet.datastructure.team_lineup.TeamSheet': ( 'dataStrcuture/team_lineup.html#teamsheet',
                                                                                                    'fastbet/datastructure/team_lineup.py'),
                                                   'fastbet.datastructure.team_lineup.TeamSheet.get_latest': ( 'dataStrcuture/team_lineup.html#teamsheet.get_latest',
                                                                                                               'fastbet/datastructure/team_lineup.py')},
            'fastbet.environment': { 'fastbet.environment.BettingEnv': ( 'Environment/betting_env.html#bettingenv',
                                                                         'fastbet/environment.py'),
                                     'fastbet.environment.BettingEnv.__init__': ( 'Environment/betting_env.html#bettingenv.__init__',
                                                                                  'fastbet/environment.py'),
                                     'fastbet.environment.BettingEnv._get_current_index': ( 'Environment/betting_env.html#bettingenv._get_current_index',
                                                                                            'fastbet/environment.py'),
                                     'fastbet.environment.BettingEnv.create_info': ( 'Environment/betting_env.html#bettingenv.create_info',
                                                                                     'fastbet/environment.py'),
                                     'fastbet.environment.BettingEnv.get_bet': ( 'Environment/betting_env.html#bettingenv.get_bet',
                                                                                 'fastbet/environment.py'),
                                     'fastbet.environment.BettingEnv.get_observation': ( 'Environment/betting_env.html#bettingenv.get_observation',
                                                                                         'fastbet/environment.py'),
                                     'fastbet.environment.BettingEnv.get_odds': ( 'Environment/betting_env.html#bettingenv.get_odds',
                                                                                  'fastbet/environment.py'),
                                     'fastbet.environment.BettingEnv.render': ( 'Environment/betting_env.html#bettingenv.render',
                                                                                'fastbet/environment.py'),
                                     'fastbet.environment.BettingEnv.reset': ( 'Environment/betting_env.html#bettingenv.reset',
                                                                               'fastbet/environment.py'),
                                     'fastbet.environment.BettingEnv.step': ( 'Environment/betting_env.html#bettingenv.step',
                                                                              'fastbet/environment.py'),
                                     'fastbet.environment.Observation': ( 'Environment/betting_env.html#observation',
                                                                          'fastbet/environment.py'),
                                     'fastbet.environment.Observation.__call__': ( 'Environment/betting_env.html#observation.__call__',
                                                                                   'fastbet/environment.py'),
                                     'fastbet.environment.Observation.__init__': ( 'Environment/betting_env.html#observation.__init__',
                                                                                   'fastbet/environment.py'),
                                     'fastbet.environment.Observation.astype': ( 'Environment/betting_env.html#observation.astype',
                                                                                 'fastbet/environment.py'),
                                     'fastbet.environment.Observation.pretty': ( 'Environment/betting_env.html#observation.pretty',
                                                                                 'fastbet/environment.py'),
                                     'fastbet.environment.Observation.reshape': ( 'Environment/betting_env.html#observation.reshape',
                                                                                  'fastbet/environment.py')},
            'fastbet.utils.asian_1x2_pnl': { 'fastbet.utils.asian_1x2_pnl.pnl': ( 'Utils/pnl_calculator.html#pnl',
                                                                                  'fastbet/utils/asian_1x2_pnl.py'),
                                             'fastbet.utils.asian_1x2_pnl.pnl_1X2': ( 'Utils/pnl_calculator.html#pnl_1x2',
                                                                                      'fastbet/utils/asian_1x2_pnl.py'),
                                             'fastbet.utils.asian_1x2_pnl.pnl_ah': ( 'Utils/pnl_calculator.html#pnl_ah',
                                                                                     'fastbet/utils/asian_1x2_pnl.py')}}}
